import os

import bonobo
from bonobo.util.iterators import tuplize
from bonobo_docker import settings
from bonobo_docker.logging import logger


def run_docker(*args):
    cmd = ' '.join(('docker', ) + args)
    logger.info(cmd)
    return os.system(cmd)


def get_image():
    return '{}:{}'.format(settings.IMAGE, bonobo.__version__)


def get_volumes(*, with_local_packages=False):
    cache_path = os.path.expanduser('~/.cache')
    volumes = {}
    volumes[cache_path] = {'bind': '/home/bonobo/.cache'}
    if with_local_packages:
        from bonobo.util.pkgs import bonobo_packages
        for name in bonobo_packages:
            volumes[bonobo_packages[name].location] = {'bind': '/home/bonobo/src/' + name}
    return volumes


@tuplize
def get_volumes_args(*, with_local_packages=False):
    for hostpath, volumespec in get_volumes(with_local_packages=with_local_packages).items():
        yield '-v {}:{}{}'.format(hostpath, volumespec['bind'], ':ro' if volumespec.get('mode') == 'ro' else '')



import os

from bonobo.commands.run import register_generic_run_arguments


def register(parser):
    register_generic_run_arguments(parser, required=False)
    parser.add_argument('--with-local-packages', '-L', action='store_true')
    parser.add_argument('--volume', '-v', action='append', dest='volumes')
    parser.add_argument('--shell', action='store_true')
    return execute


def execute(filename, module, volumes=None, shell=False, with_local_packages=False):
    from bonobo_docker.utils import run_docker, get_volumes_args, get_image

    site_volumes = get_volumes_args(with_local_packages=with_local_packages)

    if shell:
        command = "bash"
    elif filename:
        target = os.path.realpath(os.path.join(os.getcwd(), filename))

        if os.path.isdir(target):
            site_volumes += ('-v ' + target + ':/home/bonobo/app', )
            command = "bin/bonobo run --install app"
        elif os.path.isfile(target):
            site_volumes += ('-v ' + os.path.dirname(target) + ':/home/bonobo/app', )
            command = "bin/bonobo run app/" + os.path.basename(target)
        else:
            raise IOError(
                'File does not exist, or is of unsupported type (only directories and regular files are supported).'
            )
    elif module:
        raise NotImplementedError('Executing a module within a docker container is not yet implemented.')
    elif shell:
        command = '/bin/bash'
    else:
        command = '/home/bonobo/bin/python'

    run_docker(
        'run -it --rm',
        *site_volumes,
        *(['-v {}'.format(v) for v in volumes] if volumes else []),
        get_image(),
        command,
    )



# ~/.pythonrc


def _enable_syntax_completion():
    try:
        import readline
    except ImportError:
        print("Module readline not available.")
    else:
        import rlcompleter
        readline.parse_and_bind("tab: complete")


def _enable_shell_colors():
    import sys
    from colorama import Fore, Style
    sys.ps1 = Fore.LIGHTWHITE_EX + '🐵 >' + Fore.RESET + ' '
    sys.ps2 = Fore.BLACK + '..' + Fore.LIGHTBLACK_EX + '.' + Fore.RESET + ' '


_enable_syntax_completion()
_enable_shell_colors()
del _enable_syntax_completion
del _enable_shell_colors

# Preload bonobo
bonobo = __import__('bonobo')



import argparse
import re

import requests

from bonobo_docker import images, settings
from bonobo_docker.utils import run_docker

VERSION_RE = re.compile('bonobo-((?:[\d]+\.)+[\d+])-')


class BaseCommand:
    def __init__(self, parser):
        self.init(parser)

    def init(self, parser):
        pass

    def __call__(self, *args, **kwargs):
        return self.call(*args, **kwargs)

    def call(self, *args, **kwargs):
        raise NotImplementedError('Abstract.')


class BuildCommand(BaseCommand):
    def init(self, parser):
        parser.add_argument(
            '--version',
            '-v',
        )
        parser.add_argument('--push', '-p', action='store_true')
        parser.add_argument('--no-cache', action='store_true')

    def call(self, version=None, push=False, no_cache=False):
        # role : build docker images for bonobo
        # xxx todo add cache

        html = requests.get('https://pypi.python.org/simple/bonobo/')
        versions = list(sorted(set(VERSION_RE.findall(html.text)), key=lambda s: list(map(int, s.split('.')))))

        if version is None:
            version = versions[-1]

        if version not in versions:
            raise RuntimeError('Unknown version {}.'.format(version))

        tag, tags = version, {version}

        if version == versions[-1]:
            tags.add('latest')

        while '.' in tag:
            tag = tag[:tag.rfind('.')]
            matching_versions = list(filter(lambda x: x.startswith(tag), versions))

            if version == matching_versions[-1]:
                tags.add(tag)
            else:
                break

        tags = list(sorted(tags))

        run_docker(
            'build',
            *(['--no-cache'] if no_cache else []), '--build-arg BONOBO_VERSION={}'.format(version),
            *('-t {}:{}'.format(settings.IMAGE, tag) for tag in tags), images.__path__[0]
        )

        if push:
            for tag in tags:
                run_docker('push {}:{}'.format(settings.IMAGE, tag))


COMMANDS = {'build': BuildCommand}


def entrypoint(args=None):
    """
    Entrypoint for python -m bonobo-docker command. Only used for image plumbery and should not be userland
    """
    parser = argparse.ArgumentParser()

    subparsers = parser.add_subparsers(dest='command')
    subparsers.required = True

    commands = {}

    for name, command in COMMANDS.items():
        subparser = subparsers.add_parser(name)
        commands[name] = command(subparser)

    args = parser.parse_args(args).__dict__
    commands[args.pop('command')](**args)


if __name__ == '__main__':
    entrypoint()
